# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import configobj
import logging
import requests

from os import path
from phabletutils import cdimage
from xdg.BaseDirectory import xdg_config_home

log = logging.getLogger()


def get_download_dir_full_path(subdir):
    try:
        userdirs_file = path.join(xdg_config_home, 'user-dirs.dirs')
        userdirs_config = configobj.ConfigObj(userdirs_file, encoding='utf-8')
        userdirs_download = path.expandvars(
            userdirs_config['XDG_DOWNLOAD_DIR'])
        download_dir = userdirs_download
    except KeyError:
        download_dir = path.expandvars('$HOME')
        log.warning('XDG_DOWNLOAD_DIR could not be read')
    return path.join(download_dir, subdir)


def get_ubuntu_stamp(uri):
    '''Downloads the jenkins build id from stamp file'''
    try:
        ubuntu_stamp = requests.get('%s/quantal-ubuntu_stamp' % uri)
        if ubuntu_stamp.status_code != 200:
            ubuntu_stamp = requests.get('%s/ubuntu_stamp' % uri)
        if ubuntu_stamp.status_code != 200:
            log.error('Latest build detection not supported... bailing')
            exit(1)
        # Make list and get rid of empties
        build_data = filter(lambda x: x.startswith('JENKINS_BUILD='),
                            ubuntu_stamp.content.split('\n'))
        jenkins_build_id = build_data[0].split('=')[1]
    except (requests.HTTPError, requests.Timeout, requests.ConnectionError):
        log.error('Could not download build data from jenkins... bailing')
        exit(1)
    except IndexError:
        log.error('Jenkins data format has changed, incompatible')
        exit(1)
    return jenkins_build_id


class Project(object):

    @property
    def uri(self):
        return self._uri

    @property
    def download_uri(self):
        return self._download_uri

    @property
    def download_dir(self):
        return self._download_dir

    @property
    def name(self):
        return self._project

    @property
    def series(self):
        return self._series

    @property
    def hashes(self):
        return cdimage.get_sha256_dict(self._hash_content)

    @property
    def hash_content(self):
        return self._hash_content

    @property
    def hash_file_name(self):
        return self._hash_file_name

    def list_revisions(self):
        revisions = cdimage.get_available_revisions(self._uri)
        cdimage.display_revisions(revisions)

    def __init__(self, project, series, cdimage_uri_base, base_path,
                 hash_file_name, revision=None, latest_revision=None):
        if not project:
            return None
        log.debug('project: %s, series: %s, cdimage_uri_base: %s, '
                  'base_path: %s, hash_file_name: %s, revision: %s, '
                  'latest_revision: %s' % (project, series, cdimage_uri_base,
                  base_path, hash_file_name, revision, latest_revision))
        self._project = project
        self._uri = '%s/%s' % (cdimage_uri_base, project)
        self._series = series
        self._hash_file_name = hash_file_name

        if base_path:
            download_uri = None
            download_dir = base_path
        elif latest_revision:
            self._series, revision = \
                cdimage.get_latest_revision(self._uri)
            download_uri = '%s/%s/%s' % (self._uri,
                                         self._series, revision)
            download_dir = '%s/%s/%s' % (project, self._series, revision)
        elif revision:
            revision_split = revision.split('/')
            if len(revision_split) != 2:
                raise EnvironmentError(
                    'Improper use of revision, needs to be formatted like'
                    '[series]/[revision]. Use --list-revisions to find'
                    'the available revisions on cdimage')
            # TODO add verification that uri exists
            self._series = revision_split[0]
            download_uri = '%s/%s' % (self._uri, revision)
            download_dir = '%s/%s/%s' % (project, self._series, revision)
        else:
            uri = '%s/daily-preinstalled' % self._uri
            current = cdimage.get_latest_current(uri)
            download_uri = '%s/%s' % (uri, current)
            download_dir = '%s/%s' % (project, current)
        self._download_uri = download_uri
        self._download_dir = download_dir
        self._setup_hashes()

    def _setup_hashes(self):
        self._hash_path = path.join(self._download_dir, self._hash_file_name)
        if path.exists(self._hash_path):
            with open(self._hash_path, 'r') as f:
                hash_content = f.read()
        else:
            hash_content = cdimage.get_sha256_content(
                '%s/%s' % (self._download_uri, self._hash_file_name))
        self._hash_content = hash_content


class Environment(object):
    '''All the hacks to support multiple environments live here.'''

    @property
    def project(self):
        return self._project

    @property
    def download_dir(self):
        return self._download_dir

    @property
    def download_uri(self):
        return self._download_uri

    @property
    def bootstrap_files(self):
        return self._bootstrap_files

    @property
    def recovery_files(self):
        return self._recovery_files

    @property
    def device_zip(self):
        return self._device_zip

    @property
    def device_zip_path(self):
        if self._device_zip:
            return path.join(self._download_dir, self._device_zip)
        else:
            return None

    @property
    def ubuntu_zip(self):
        return self._ubuntu_zip

    @property
    def ubuntu_zip_path(self):
        if self._ubuntu_zip:
            return path.join(self._download_dir, self._ubuntu_zip)
        else:
            return None

    @property
    def system_img(self):
        return self._device_img

    @property
    def system_img_path(self):
        return path.join(self._download_dir, self._system_img)

    @property
    def boot_img(self):
        return self._boot_img

    @property
    def boot_img_path(self):
        return path.join(self._download_dir, self._boot_img)

    @property
    def recovery_img(self):
        return self._recovery_img

    @property
    def recovery_img_path(self):
        if self._recovery_img:
            return path.join(self._download_dir, self._recovery_img)
        else:
            return None

    @property
    def offline(self):
        return self._offline

    def __init__(self, preset_uri, device, series, latest_revision, revision,
                 legacy, base_path, settings):
        self._offline = False
        self._set_project(series, legacy, base_path, latest_revision,
                          revision, settings)
        self._set_download(preset_uri, settings)
        self._set_files(device, settings)

    def _set_project(self, series, legacy, base_path,
                     latest_revision, revision,
                     settings):
        if getattr(settings, 'revision', None):
            if not series:
                series = settings.default_series
            if legacy:
                project_name = 'ubuntu-touch-preview'
            else:
                project_name = 'ubuntu-touch'
            self._project = Project(
                project_name, series, settings.cdimage_uri_base, base_path,
                settings.hash_file_name, latest_revision, revision)
        else:
            self._project = None

    def _set_download(self, preset_uri, settings):
        if preset_uri:
            uri = preset_uri
            download_dir = path.join(settings.download_dir,
                                     get_ubuntu_stamp(uri))
        elif not self._project:
            uri = settings.download_uri
            download_dir = path.join(settings.download_dir,
                                     get_ubuntu_stamp(uri))
        elif self._project:
            uri = self._project.download_uri
            download_dir = path.join(settings.download_dir,
                                     self._project.download_dir)
        else:
            raise EnvironmentError('Environment not setup correctly')
        log.debug('Download set to %s' % uri)
        self._download_uri = uri
        self._download_dir = get_download_dir_full_path(download_dir)

    def _set_files(self, device, settings):
        if self._project:
            project = self._project.name
            self._ubuntu_zip = settings.files[project]['ubuntu_zip'] % \
                (self._project.series,)
            self._device_zip = settings.files[project]['device_zip'] % \
                (self._project.series, device)
            self._system_img = settings.files[project]['system_img'] % \
                (self._project.series, device)
            self._boot_img = settings.files[project]['boot_img'] % \
                (self._project.series, device)
            self._recovery_img = settings.files[project]['recovery_img'] % \
                (self._project.series, device)
            self._bootstrap_files = [
                [self._ubuntu_zip, self.ubuntu_zip_path],
                [self._system_img, self.system_img_path],
                [self._boot_img, self.boot_img_path],
                [self._recovery_img, self.recovery_img_path]]
            self._recovery_files = [
                [self._ubuntu_zip, self.ubuntu_zip_path],
                [self._device_zip, self.device_zip_path]]
        else:
            self._ubuntu_zip = None
            self._device_zip = None
            self._recovery_img = None
            self._system_img = settings.device_file_img % (device,)
            self._boot_img = settings.boot_file_img % (device,)
            self._bootstrap_files = [
                [self._system_img, self.system_img_path],
                [self._boot_img, self.boot_img_path]]

    def store_hashes(self):
        if not self._project.hash_content:
            return None
        with open(path.join(self._download_dir,
                  self._project.hash_file_name), 'w') as f:
            for line in self._project.hash_content:
                f.write(line)
